"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImmutableXLinkConnectionProvider = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var immutable_wallet_1 = require("@rarible/immutable-wallet");
var connector_1 = require("@rarible/connector");
var PROVIDER_ID = "immutablex";
var ImmutableXLinkConnectionProvider = /** @class */ (function (_super) {
    tslib_1.__extends(ImmutableXLinkConnectionProvider, _super);
    function ImmutableXLinkConnectionProvider(config) {
        var _this = _super.call(this) || this;
        _this.config = config;
        _this.instance = (0, connector_1.cache)(function () { return _this._connect(); });
        _this.connection = (0, rxjs_1.defer)(function () { return _this.instance.pipe((0, operators_1.mergeMap)(function (instance) { return _this.toConnectState(instance.wallet); }), (0, operators_1.startWith)((0, connector_1.getStateConnecting)({ providerId: PROVIDER_ID }))); });
        return _this;
    }
    ImmutableXLinkConnectionProvider.prototype._connect = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var wallet;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        wallet = new immutable_wallet_1.ImxWallet(this.config.env);
                        if (!(wallet.getConnectionData().status !== "connected")) return [3 /*break*/, 2];
                        return [4 /*yield*/, wallet.connect()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/, { wallet: wallet }];
                }
            });
        });
    };
    ImmutableXLinkConnectionProvider.prototype.getId = function () {
        return PROVIDER_ID;
    };
    ImmutableXLinkConnectionProvider.prototype.getAddress = function (wallet) {
        return new rxjs_1.Observable(function (subscriber) {
            subscriber.next(wallet.getConnectionData().address);
            // wallet.link.syncState({}).then((stream) => {
            // 	stream.subscribe((event) => {
            // 		console.log(event)
            // 		if (event.eventType === SyncStateEventTypes.WALLET_CHANGE) {
            // 			// if ((event as ActiveWalletChangeSyncStateEvent).connectedWalletAddress !== null) {
            // 			// 	subscriber.next((event as ActiveWalletChangeSyncStateEvent).connectedWalletAddres)
            // 			// }
            // 		}
            // 	})
            // })
        });
    };
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ImmutableXLinkConnectionProvider.prototype.getConnectedStatus = function (_wallet) {
        return new rxjs_1.Observable(function (subscriber) {
            subscriber.next("connected");
            // wallet.link.syncState({}).then((stream) => {
            // 	stream.subscribe((event) => {
            // 		if (event.eventType === SyncStateEventTypes.DISCONNECT) {
            // 			subscriber.next("disconnected")
            // 		}
            // 	})
            // })
        });
    };
    ImmutableXLinkConnectionProvider.prototype.toConnectState = function (wallet) {
        var _this = this;
        return (0, rxjs_1.combineLatest)([
            this.getAddress(wallet),
            this.getConnectedStatus(wallet),
        ]).pipe((0, operators_1.distinctUntilChanged)(function (c1, c2) {
            return c1[0] === c2[0] && c1[1] === c2[1];
        }), (0, operators_1.map)(function (_a) {
            var _b = tslib_1.__read(_a, 2), address = _b[0], status = _b[1];
            if (status === "connected" && address) {
                var connection = {
                    address: address,
                    wallet: wallet,
                };
                return (0, connector_1.getStateConnected)({
                    connection: connection,
                    disconnect: function () { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, wallet.disconnect()];
                            case 1: return [2 /*return*/, _a.sent()];
                        }
                    }); }); },
                });
            }
            else {
                return (0, connector_1.getStateDisconnected)();
            }
        }));
    };
    ImmutableXLinkConnectionProvider.prototype.getConnection = function () {
        return this.connection;
    };
    ImmutableXLinkConnectionProvider.prototype.getOption = function () {
        return Promise.resolve(PROVIDER_ID);
    };
    ImmutableXLinkConnectionProvider.prototype.isAutoConnected = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, false];
            });
        });
    };
    ImmutableXLinkConnectionProvider.prototype.isConnected = function () {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.instance.pipe((0, operators_1.first)()).toPromise()];
                    case 1: return [2 /*return*/, ((_a = (_b.sent())) === null || _a === void 0 ? void 0 : _a.wallet.getConnectionData().status) === "connected"];
                }
            });
        });
    };
    return ImmutableXLinkConnectionProvider;
}(connector_1.AbstractConnectionProvider));
exports.ImmutableXLinkConnectionProvider = ImmutableXLinkConnectionProvider;
