"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cancel = exports.buy = exports.sell = void 0;
var tslib_1 = require("tslib");
var ethereum_api_client_1 = require("@rarible/ethereum-api-client");
var convert_fees_1 = require("../common/convert-fees");
var env_1 = require("../config/env");
var trades_1 = require("../apis/trades");
var utils_1 = require("../common/utils");
function sell(link, request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, tokenId, contract, takeAssetType, amount, payouts, originFees, currencyContract, orderId;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (link === undefined) {
                        throw new Error("Wallet undefined");
                    }
                    _a = request.makeAssetType, tokenId = _a.tokenId, contract = _a.contract, takeAssetType = request.takeAssetType, amount = request.amount, payouts = request.payouts, originFees = request.originFees;
                    currencyContract = takeAssetType.assetClass === "ERC20" ? takeAssetType.contract : undefined;
                    return [4 /*yield*/, link.sell(tslib_1.__assign({ tokenId: tokenId, tokenAddress: contract, fees: (0, convert_fees_1.convertFees)(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(payouts), false), tslib_1.__read(originFees), false)), amount: amount }, currencyContract ? { currencyAddress: currencyContract } : {}))];
                case 1:
                    orderId = _b.sent();
                    return [2 /*return*/, {
                            orderId: orderId,
                        }];
            }
        });
    });
}
exports.sell = sell;
function buy(env, link, request, token) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var orderId, fee, buyResult, tradeResult, txId, e_1;
        var _this = this;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (link === undefined) {
                        throw new Error("Wallet undefined");
                    }
                    orderId = request.orderId, fee = request.fee;
                    return [4 /*yield*/, link.buy({
                            orderIds: [orderId],
                            fees: (0, convert_fees_1.convertFees)(fee),
                        })];
                case 1:
                    buyResult = _b.sent();
                    tradeResult = (_a = buyResult.result) === null || _a === void 0 ? void 0 : _a[orderId];
                    if (tradeResult.status === "error") {
                        throw new Error("Trade unsuccessful: " + tradeResult.message);
                    }
                    txId = undefined;
                    _b.label = 2;
                case 2:
                    _b.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, (0, utils_1.retry)(20, 2000, function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var apiAddressV1, balanceApiConfig, api, result, trade;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        apiAddressV1 = env_1.IMX_ENV_CONFIG[env].apiAddressV1;
                                        balanceApiConfig = new ethereum_api_client_1.Configuration({ basePath: apiAddressV1 });
                                        api = new trades_1.ImxTradesControllerApi(balanceApiConfig);
                                        return [4 /*yield*/, api.getTrades({
                                                tokenType: token.assetClass,
                                                tokenAddress: token.contract,
                                                tokenId: token.tokenId,
                                            })];
                                    case 1:
                                        result = (_a.sent()).result;
                                        trade = result.find(function (trade) { return trade.b.order_id.toString() === orderId; });
                                        if (!trade) {
                                            throw new Error("Can't find trade with orderid " + orderId);
                                        }
                                        if (tradeResult.status === "success") {
                                            return [2 /*return*/, trade.transaction_id];
                                        }
                                        return [2 /*return*/];
                                }
                            });
                        }); })];
                case 3:
                    txId = _b.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_1 = _b.sent();
                    console.error("Error while fetching trade", e_1);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, {
                        txId: txId,
                    }];
            }
        });
    });
}
exports.buy = buy;
function cancel(link, request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var orderId, result;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (link === undefined) {
                        throw new Error("Wallet undefined");
                    }
                    orderId = request.orderId;
                    return [4 /*yield*/, link.cancel({
                            orderId: orderId,
                        })];
                case 1:
                    result = _a.sent();
                    return [2 /*return*/, {
                            orderId: result.order_id.toString(),
                        }];
            }
        });
    });
}
exports.cancel = cancel;
