"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IMX_ENV_CONFIG = exports.IMX_NETWORK_CONFIG = exports.ImxWallet = void 0;
var tslib_1 = require("tslib");
var imx_sdk_1 = require("@imtbl/imx-sdk");
var domain_1 = require("./domain");
var config_1 = require("./config");
Object.defineProperty(exports, "IMX_ENV_CONFIG", { enumerable: true, get: function () { return config_1.IMX_ENV_CONFIG; } });
Object.defineProperty(exports, "IMX_NETWORK_CONFIG", { enumerable: true, get: function () { return config_1.IMX_NETWORK_CONFIG; } });
var ImxWallet = /** @class */ (function () {
    function ImxWallet(env, provider, iframeOptions, storage) {
        if (provider === void 0) { provider = domain_1.ImxWalletProviderEnum.NONE; }
        if (storage === void 0) { storage = localStorage; }
        var _this = this;
        this.env = env;
        this.provider = provider;
        this.iframeOptions = iframeOptions;
        this.storage = storage;
        this.state = Object.keys(StorageKeyEnum).reduce(function (prev, curr) {
            var _a;
            return (tslib_1.__assign(tslib_1.__assign({}, prev), (_a = {}, _a[curr] = _this.storage.getItem(curr) || "", _a)));
        }, {});
        this.status = this.state[StorageKeyEnum.IMX_ADDRESS] ? "connected" : "disconnected";
        this.network = tslib_1.__assign(tslib_1.__assign({}, config_1.IMX_ENV_CONFIG[this.env]), { env: this.env });
        this.link = new imx_sdk_1.Link(this.network.linkAddress, this.iframeOptions);
        this.connect = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var data, error_1;
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.link.setup(this.getSetupOptions())];
                    case 1:
                        data = _b.sent();
                        if (data.address && data.starkPublicKey) {
                            this.status = "connected";
                            this.state = (_a = {},
                                _a[StorageKeyEnum.IMX_ADDRESS] = data.address,
                                _a[StorageKeyEnum.IMX_STARK_KEY] = data.starkPublicKey,
                                _a[StorageKeyEnum.IMX_ETH_NETWORK] = data.ethNetwork,
                                _a[StorageKeyEnum.IMX_PROVIDER] = data.providerPreference,
                                _a);
                            this.setValue(StorageKeyEnum.IMX_ADDRESS, data.address);
                            this.setValue(StorageKeyEnum.IMX_STARK_KEY, data.starkPublicKey);
                            this.setValue(StorageKeyEnum.IMX_ETH_NETWORK, data.ethNetwork);
                            this.setValue(StorageKeyEnum.IMX_PROVIDER, data.providerPreference);
                            return [2 /*return*/, data];
                        }
                        throw new Error("There is no address or starkAddress in response");
                    case 2:
                        error_1 = _b.sent();
                        console.error("Connection failed with reason", error_1);
                        throw error_1;
                    case 3: return [2 /*return*/];
                }
            });
        }); };
        this.disconnect = function () {
            Object.keys(StorageKeyEnum).forEach(function (x) { return _this.setValue(x, ""); });
            _this.status = "disconnected";
        };
        this.getConnectionData = function () { return ({
            address: _this.state[StorageKeyEnum.IMX_ADDRESS],
            starkPublicKey: _this.state[StorageKeyEnum.IMX_STARK_KEY],
            ethNetwork: _this.state[StorageKeyEnum.IMX_ETH_NETWORK],
            providerPreference: _this.state[StorageKeyEnum.IMX_PROVIDER],
            link: _this.link,
            status: _this.status,
        }); };
    }
    ImxWallet.prototype.getSetupOptions = function () {
        return this.provider ? { providerPreference: imx_sdk_1.ProviderPreference[this.provider] } : {};
    };
    ImxWallet.prototype.setValue = function (key, value) {
        this.state[key] = value;
        return this.storage.setItem(key, value);
    };
    return ImxWallet;
}());
exports.ImxWallet = ImxWallet;
var StorageKeyEnum;
(function (StorageKeyEnum) {
    StorageKeyEnum["IMX_STARK_KEY"] = "IMX_STARK_KEY";
    StorageKeyEnum["IMX_ADDRESS"] = "IMX_ADDRESS";
    StorageKeyEnum["IMX_PROVIDER"] = "IMX_PROVIDER";
    StorageKeyEnum["IMX_ETH_NETWORK"] = "IMX_ETH_NETWORK";
})(StorageKeyEnum || (StorageKeyEnum = {}));
