"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createImmutablexSdk = void 0;
var immutable_sdk_1 = require("@rarible/immutable-sdk");
var not_implemented_1 = require("../../common/not-implemented");
var nft_1 = require("./nft");
var order_1 = require("./order");
var balance_1 = require("./balance");
function createImmutablexSdk(wallet, apis, env) {
    var sdk = (0, immutable_sdk_1.createImxSdk)(wallet === null || wallet === void 0 ? void 0 : wallet.wallet.link, env);
    var nftService = new nft_1.ImxNftService(sdk, apis);
    var orderService = new order_1.ImxOrderService(sdk, apis);
    var balancesService = new balance_1.ImxBalanceService(sdk, apis);
    return {
        nft: {
            mint: not_implemented_1.notImplemented,
            burn: nftService.burn,
            transfer: nftService.transfer,
            generateTokenId: not_implemented_1.notImplemented,
            deploy: not_implemented_1.nonImplementedAction,
            createCollection: not_implemented_1.nonImplementedAction,
            preprocessMeta: not_implemented_1.notImplemented,
            uploadMeta: not_implemented_1.notImplemented,
        },
        order: {
            fill: orderService.buy,
            buy: orderService.buy,
            batchBuy: not_implemented_1.nonImplementedAction,
            acceptBid: not_implemented_1.notImplemented,
            sell: orderService.sell,
            sellUpdate: not_implemented_1.notImplemented,
            bid: not_implemented_1.notImplemented,
            bidUpdate: not_implemented_1.notImplemented,
            cancel: orderService.cancel,
        },
        balances: {
            getBalance: balancesService.getBalance,
            convert: not_implemented_1.notImplemented,
            getBiddingBalance: not_implemented_1.nonImplementedAction,
            depositBiddingBalance: not_implemented_1.nonImplementedAction,
            withdrawBiddingBalance: not_implemented_1.nonImplementedAction,
        },
        restriction: {
            canTransfer: not_implemented_1.notImplemented,
        },
    };
}
exports.createImmutablexSdk = createImmutablexSdk;
