"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImxOrderService = void 0;
var tslib_1 = require("tslib");
var sdk_transaction_1 = require("@rarible/sdk-transaction");
var types_1 = require("@rarible/types");
var api_client_1 = require("@rarible/api-client");
var action_1 = require("@rarible/action");
var domain_1 = require("../../types/order/fill/domain");
var utils_1 = require("./common/utils");
var currencies_1 = require("./common/currencies");
var ImxOrderService = /** @class */ (function () {
    function ImxOrderService(sdk, apis) {
        var _this = this;
        this.sdk = sdk;
        this.apis = apis;
        this.cancel = action_1.Action.create({
            id: "send-tx",
            run: function (request) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var _a, orderId;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            _a = tslib_1.__read(request.orderId.split(":"), 2), orderId = _a[1];
                            return [4 /*yield*/, this.sdk.order.cancel({
                                    orderId: orderId,
                                })];
                        case 1:
                            _b.sent();
                            return [2 /*return*/, new sdk_transaction_1.BlockchainImmutableXTransaction(undefined)];
                    }
                });
            }); },
        });
        this.sell = this.sell.bind(this);
        this.buy = this.buy.bind(this);
    }
    ImxOrderService.prototype.sell = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var submit;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                submit = action_1.Action.create({
                    id: "send-tx",
                    run: function (request) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var _a, contract, tokenId, res;
                        return tslib_1.__generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    _a = tslib_1.__read(request.itemId.split(":"), 3), contract = _a[1], tokenId = _a[2];
                                    return [4 /*yield*/, this.sdk.order.sell({
                                            amount: (0, types_1.toBigNumber)(request.price.toString()),
                                            originFees: (0, utils_1.unionPartsToParts)(request.originFees),
                                            payouts: (0, utils_1.unionPartsToParts)(request.payouts),
                                            makeAssetType: {
                                                assetClass: "ERC721",
                                                contract: (0, types_1.toAddress)(contract),
                                                tokenId: (0, types_1.toBigNumber)(tokenId),
                                            },
                                            takeAssetType: (0, utils_1.getTakeAssetType)(request.currency),
                                        })];
                                case 1:
                                    res = _b.sent();
                                    return [2 /*return*/, (0, types_1.toOrderId)("IMMUTABLEX:" + res.orderId)];
                            }
                        });
                    }); },
                });
                return [2 /*return*/, {
                        originFeeSupport: domain_1.OriginFeeSupport.FULL,
                        payoutsSupport: domain_1.PayoutsSupport.MULTIPLE,
                        maxFeesBasePointSupport: domain_1.MaxFeesBasePointSupport.IGNORED,
                        supportedCurrencies: (0, currencies_1.getCurrencies)(),
                        baseFee: 200,
                        supportsExpirationDate: false,
                        submit: submit,
                    }];
            });
        });
    };
    ImxOrderService.prototype.buy = function (prepare) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var order, getERC721Asset, submit;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (0, utils_1.getPreparedOrder)(prepare, this.apis)];
                    case 1:
                        order = _a.sent();
                        if (order.status !== api_client_1.OrderStatus.ACTIVE) {
                            throw new Error("Order is not active");
                        }
                        getERC721Asset = function () {
                            if (order.make.type["@type"] !== "ERC721") {
                                throw new Error("Order make type should be ERC721");
                            }
                            var _a = tslib_1.__read(order.make.type.contract.split(":"), 2), address = _a[1];
                            return {
                                assetClass: "ERC721",
                                contract: address,
                                tokenId: order.make.type.tokenId,
                            };
                        };
                        submit = action_1.Action
                            .create({
                            id: "send-tx",
                            run: function (request) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var _a, orderId, res;
                                return tslib_1.__generator(this, function (_b) {
                                    switch (_b.label) {
                                        case 0:
                                            _a = tslib_1.__read(order.id.split(":"), 2), orderId = _a[1];
                                            return [4 /*yield*/, this.sdk.order.buy({
                                                    orderId: orderId,
                                                    fee: (0, utils_1.unionPartsToParts)(request.originFees),
                                                }, getERC721Asset())];
                                        case 1:
                                            res = _b.sent();
                                            console.log(res);
                                            return [2 /*return*/, res];
                                    }
                                });
                            }); },
                        })
                            .after(function (res) { return new sdk_transaction_1.BlockchainImmutableXTransaction(res.txId); });
                        return [2 /*return*/, {
                                multiple: false,
                                maxAmount: order.makeStock,
                                baseFee: (0, utils_1.calcBuyerBaseFee)(order),
                                supportsPartialFill: false,
                                maxFeesBasePointSupport: domain_1.MaxFeesBasePointSupport.IGNORED,
                                originFeeSupport: domain_1.OriginFeeSupport.FULL,
                                payoutsSupport: domain_1.PayoutsSupport.NONE,
                                submit: submit,
                            }];
                }
            });
        });
    };
    return ImxOrderService;
}());
exports.ImxOrderService = ImxOrderService;
