"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTokenAmount = exports.getAssociatedTokenAccountForMint = exports.getAccountInfo = exports.getPriceWithMantissa = exports.getMasterEdition = exports.getMetadata = exports.createAssociatedTokenAccountInstruction = exports.getTokenWallet = void 0;
var tslib_1 = require("tslib");
var web3_js_1 = require("@solana/web3.js");
var spl_token_1 = require("@solana/spl-token");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var contracts_1 = require("./contracts");
function getTokenWallet(wallet, mint) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, web3_js_1.PublicKey.findProgramAddress([wallet.toBuffer(), spl_token_1.TOKEN_PROGRAM_ID.toBuffer(), mint.toBuffer()], contracts_1.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID)];
                case 1: return [2 /*return*/, (_a.sent())[0]];
            }
        });
    });
}
exports.getTokenWallet = getTokenWallet;
function createAssociatedTokenAccountInstruction(associatedTokenAddress, payer, walletAddress, splTokenMintAddress) {
    var keys = [
        {
            pubkey: payer,
            isSigner: true,
            isWritable: true,
        },
        {
            pubkey: associatedTokenAddress,
            isSigner: false,
            isWritable: true,
        },
        {
            pubkey: walletAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: splTokenMintAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: web3_js_1.SystemProgram.programId,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: spl_token_1.TOKEN_PROGRAM_ID,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
            isSigner: false,
            isWritable: false,
        },
    ];
    return new web3_js_1.TransactionInstruction({
        keys: keys,
        programId: contracts_1.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID,
        data: Buffer.from([]),
    });
}
exports.createAssociatedTokenAccountInstruction = createAssociatedTokenAccountInstruction;
function getMetadata(mint) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, web3_js_1.PublicKey.findProgramAddress([
                        Buffer.from("metadata"),
                        contracts_1.TOKEN_METADATA_PROGRAM_ID.toBuffer(),
                        mint.toBuffer(),
                    ], contracts_1.TOKEN_METADATA_PROGRAM_ID)];
                case 1: return [2 /*return*/, (_a.sent())[0]];
            }
        });
    });
}
exports.getMetadata = getMetadata;
function getMasterEdition(mint) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, web3_js_1.PublicKey.findProgramAddress([
                        Buffer.from("metadata"),
                        contracts_1.TOKEN_METADATA_PROGRAM_ID.toBuffer(),
                        mint.toBuffer(),
                        Buffer.from("edition"),
                    ], contracts_1.TOKEN_METADATA_PROGRAM_ID)];
                case 1: return [2 /*return*/, (_a.sent())[0]];
            }
        });
    });
}
exports.getMasterEdition = getMasterEdition;
function getPriceWithMantissa(connection, price, mint, walletKeyPair) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var token, mintInfo, mantissa;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    token = new spl_token_1.Token(connection, mint, spl_token_1.TOKEN_PROGRAM_ID, walletKeyPair);
                    return [4 /*yield*/, token.getMintInfo()];
                case 1:
                    mintInfo = _a.sent();
                    mantissa = Math.pow(10, mintInfo.decimals);
                    return [2 /*return*/, price.multipliedBy(mantissa).integerValue(bignumber_js_1.default.ROUND_CEIL)];
            }
        });
    });
}
exports.getPriceWithMantissa = getPriceWithMantissa;
function getAccountInfo(connection, mint, walletKeyPair, tokenAccount) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var token;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    token = new spl_token_1.Token(connection, mint, spl_token_1.TOKEN_PROGRAM_ID, walletKeyPair);
                    return [4 /*yield*/, token.getAccountInfo(tokenAccount)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.getAccountInfo = getAccountInfo;
function getAssociatedTokenAccountForMint(mint, publicKey) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, web3_js_1.PublicKey.findProgramAddress([
                        publicKey.toBuffer(),
                        spl_token_1.TOKEN_PROGRAM_ID.toBuffer(),
                        mint.toBuffer(),
                    ], contracts_1.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.getAssociatedTokenAccountForMint = getAssociatedTokenAccountForMint;
function getTokenAmount(connection, anchorProgram, account, mint, integer) {
    var _a;
    if (integer === void 0) { integer = false; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var amount, token, e_1, _b;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    amount = new bignumber_js_1.default(0);
                    if (!!mint.equals(contracts_1.WRAPPED_SOL_MINT)) return [3 /*break*/, 5];
                    _c.label = 1;
                case 1:
                    _c.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, connection.getTokenAccountBalance(account, "confirmed")];
                case 2:
                    token = _c.sent();
                    if ((_a = token === null || token === void 0 ? void 0 : token.value) === null || _a === void 0 ? void 0 : _a.uiAmount) {
                        amount = integer ?
                            new bignumber_js_1.default(token.value.uiAmount).multipliedBy(Math.pow(10, token.value.decimals)) :
                            new bignumber_js_1.default(token.value.uiAmount);
                    }
                    return [3 /*break*/, 4];
                case 3:
                    e_1 = _c.sent();
                    console.error(e_1);
                    console.info("Account ", account.toBase58(), "didnt return value. Assuming 0 tokens.");
                    return [3 /*break*/, 4];
                case 4: return [3 /*break*/, 7];
                case 5:
                    _b = bignumber_js_1.default.bind;
                    return [4 /*yield*/, connection.getBalance(account, "confirmed")];
                case 6:
                    amount = new (_b.apply(bignumber_js_1.default, [void 0, _c.sent()]))();
                    amount = integer ? amount : amount.dividedBy(web3_js_1.LAMPORTS_PER_SOL);
                    _c.label = 7;
                case 7: return [2 /*return*/, amount];
            }
        });
    });
}
exports.getTokenAmount = getTokenAmount;
