"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaAuctionHouseSdk = void 0;
var tslib_1 = require("tslib");
var auction_house_helpers_1 = require("../../common/auction-house-helpers");
var helpers_1 = require("../../common/helpers");
var prepared_transaction_1 = require("../prepared-transaction");
var escrow_withdraw_1 = require("./methods/escrow-withdraw");
var escrow_deposit_1 = require("./methods/escrow-deposit");
var SolanaAuctionHouseSdk = /** @class */ (function () {
    function SolanaAuctionHouseSdk(connection, logger) {
        this.connection = connection;
        this.logger = logger;
    }
    SolanaAuctionHouseSdk.prototype.getEscrowBalance = function (request) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var anchorProgram, auctionHouseObj, _a, escrowPaymentAccount, amount;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(this.connection, request.signer)];
                    case 1:
                        anchorProgram = _b.sent();
                        return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                    case 2:
                        auctionHouseObj = _b.sent();
                        return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseBuyerEscrow)(request.auctionHouse, request.wallet)];
                    case 3:
                        _a = tslib_1.__read.apply(void 0, [_b.sent(), 1]), escrowPaymentAccount = _a[0];
                        return [4 /*yield*/, (0, helpers_1.getTokenAmount)(this.connection, anchorProgram, escrowPaymentAccount, auctionHouseObj.treasuryMint)];
                    case 4:
                        amount = _b.sent();
                        this.logger.log("".concat(request.wallet.toString(), " escrow balance: ").concat(amount.toString(), " (AuctionHouse: ").concat(request.auctionHouse.toString(), ")"));
                        return [2 /*return*/, amount];
                }
            });
        });
    };
    SolanaAuctionHouseSdk.prototype.withdrawEscrow = function (request) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instructions;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (0, escrow_withdraw_1.getActionHouseEscrowWithdrawInstructions)({
                            connection: this.connection,
                            auctionHouse: request.auctionHouse,
                            signer: request.signer,
                            amount: request.amount,
                        })];
                    case 1:
                        instructions = _a.sent();
                        return [2 /*return*/, new prepared_transaction_1.PreparedTransaction(this.connection, instructions, request.signer, this.logger, function () {
                                _this.logger.log("Withdrew", request.amount, "from Auction House Escrow account");
                            })];
                }
            });
        });
    };
    SolanaAuctionHouseSdk.prototype.depositEscrow = function (request) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instructions;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (0, escrow_deposit_1.getActionHouseEscrowDepositInstructions)({
                            connection: this.connection,
                            auctionHouse: request.auctionHouse,
                            signer: request.signer,
                            amount: request.amount,
                        })];
                    case 1:
                        instructions = _a.sent();
                        return [2 /*return*/, new prepared_transaction_1.PreparedTransaction(this.connection, instructions, request.signer, this.logger, function () {
                                _this.logger.log("Deposited", request.amount, "to Auction House Escrow account");
                            })];
                }
            });
        });
    };
    return SolanaAuctionHouseSdk;
}());
exports.SolanaAuctionHouseSdk = SolanaAuctionHouseSdk;
