"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getActionHouseEscrowDepositInstructions = void 0;
var tslib_1 = require("tslib");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var web3 = tslib_1.__importStar(require("@solana/web3.js"));
var solana_wallet_1 = require("@rarible/solana-wallet");
var spl_token_1 = require("@solana/spl-token");
var contracts_1 = require("../../../common/contracts");
var helpers_1 = require("../../../common/helpers");
var auction_house_helpers_1 = require("../../../common/auction-house-helpers");
var utils_1 = require("../../../common/utils");
function getActionHouseEscrowDepositInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var walletKeyPair, anchorProgram, auctionHouseObj, amountAdjusted, _a, _b, escrowPaymentAccount, escrowBump, isNative, ata, transferAuthority, signers, instruction, instructions;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    walletKeyPair = request.signer;
                    return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(request.connection, request.signer)];
                case 1:
                    anchorProgram = _c.sent();
                    return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                case 2:
                    auctionHouseObj = _c.sent();
                    _a = utils_1.bigNumToBn;
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.amount), auctionHouseObj.treasuryMint, walletKeyPair)];
                case 3:
                    amountAdjusted = _a.apply(void 0, [_c.sent()]);
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseBuyerEscrow)(request.auctionHouse, walletKeyPair.publicKey)];
                case 4:
                    _b = tslib_1.__read.apply(void 0, [_c.sent(), 2]), escrowPaymentAccount = _b[0], escrowBump = _b[1];
                    isNative = auctionHouseObj.treasuryMint.equals(contracts_1.WRAPPED_SOL_MINT);
                    return [4 /*yield*/, (0, helpers_1.getAssociatedTokenAccountForMint)(auctionHouseObj.treasuryMint, walletKeyPair.publicKey)];
                case 5:
                    ata = (_c.sent())[0];
                    transferAuthority = solana_wallet_1.SolanaKeypairWallet.generate();
                    signers = isNative ? [] : [transferAuthority];
                    return [4 /*yield*/, anchorProgram.instruction.deposit(escrowBump, amountAdjusted, {
                            accounts: {
                                wallet: walletKeyPair.publicKey,
                                paymentAccount: isNative ? walletKeyPair.publicKey : ata,
                                transferAuthority: isNative
                                    ? web3.SystemProgram.programId
                                    : transferAuthority.publicKey,
                                escrowPaymentAccount: escrowPaymentAccount,
                                //@ts-ignore
                                treasuryMint: auctionHouseObj.treasuryMint,
                                //@ts-ignore
                                authority: auctionHouseObj.authority,
                                auctionHouse: request.auctionHouse,
                                //@ts-ignore
                                auctionHouseFeeAccount: auctionHouseObj.auctionHouseFeeAccount,
                                tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                                systemProgram: web3.SystemProgram.programId,
                                rent: web3.SYSVAR_RENT_PUBKEY,
                            },
                        })];
                case 6:
                    instruction = _c.sent();
                    if (!isNative) {
                        instruction.keys
                            .filter(function (k) { return k.pubkey.equals(transferAuthority.publicKey); })
                            .map(function (k) { return (k.isSigner = true); });
                    }
                    instructions = tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read((isNative
                        ? []
                        : [
                            spl_token_1.Token.createApproveInstruction(spl_token_1.TOKEN_PROGRAM_ID, ata, transferAuthority.publicKey, walletKeyPair.publicKey, [], amountAdjusted),
                        ])), false), [
                        instruction
                    ], false), tslib_1.__read((isNative
                        ? []
                        : [
                            spl_token_1.Token.createRevokeInstruction(spl_token_1.TOKEN_PROGRAM_ID, ata, walletKeyPair.publicKey, []),
                        ])), false);
                    return [2 /*return*/, { instructions: instructions, signers: signers }];
            }
        });
    });
}
exports.getActionHouseEscrowDepositInstructions = getActionHouseEscrowDepositInstructions;
