"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getActionHouseEscrowWithdrawInstructions = void 0;
var tslib_1 = require("tslib");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var web3 = tslib_1.__importStar(require("@solana/web3.js"));
var spl_token_1 = require("@solana/spl-token");
var contracts_1 = require("../../../common/contracts");
var helpers_1 = require("../../../common/helpers");
var auction_house_helpers_1 = require("../../../common/auction-house-helpers");
var utils_1 = require("../../../common/utils");
function getActionHouseEscrowWithdrawInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var walletKeyPair, anchorProgram, auctionHouseObj, amountAdjusted, _a, _b, escrowPaymentAccount, escrowBump, isNative, ata, signers, instruction;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    walletKeyPair = request.signer;
                    return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(request.connection, request.signer)];
                case 1:
                    anchorProgram = _c.sent();
                    return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                case 2:
                    auctionHouseObj = _c.sent();
                    _a = utils_1.bigNumToBn;
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.amount), auctionHouseObj.treasuryMint, walletKeyPair)];
                case 3:
                    amountAdjusted = _a.apply(void 0, [_c.sent()]);
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseBuyerEscrow)(request.auctionHouse, walletKeyPair.publicKey)];
                case 4:
                    _b = tslib_1.__read.apply(void 0, [_c.sent(), 2]), escrowPaymentAccount = _b[0], escrowBump = _b[1];
                    isNative = auctionHouseObj.treasuryMint.equals(contracts_1.WRAPPED_SOL_MINT);
                    return [4 /*yield*/, (0, helpers_1.getAssociatedTokenAccountForMint)(auctionHouseObj.treasuryMint, walletKeyPair.publicKey)];
                case 5:
                    ata = (_c.sent())[0];
                    signers = [];
                    return [4 /*yield*/, anchorProgram.instruction.withdraw(escrowBump, amountAdjusted, {
                            accounts: {
                                wallet: walletKeyPair.publicKey,
                                receiptAccount: isNative ? walletKeyPair.publicKey : ata,
                                escrowPaymentAccount: escrowPaymentAccount,
                                treasuryMint: auctionHouseObj.treasuryMint,
                                authority: auctionHouseObj.authority,
                                auctionHouse: request.auctionHouse,
                                auctionHouseFeeAccount: auctionHouseObj.auctionHouseFeeAccount,
                                tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                                systemProgram: web3.SystemProgram.programId,
                                rent: web3.SYSVAR_RENT_PUBKEY,
                                ataProgram: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
                            },
                            signers: signers,
                        })];
                case 6:
                    instruction = _c.sent();
                    instruction.keys
                        .filter(function (k) { return k.pubkey.equals(walletKeyPair.publicKey); })
                        .map(function (k) { return (k.isSigner = true); });
                    return [2 /*return*/, { instructions: [instruction], signers: signers }];
            }
        });
    });
}
exports.getActionHouseEscrowWithdrawInstructions = getActionHouseEscrowWithdrawInstructions;
