"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBurnTokenInstruction = exports.getTokenBurnInstructions = void 0;
var tslib_1 = require("tslib");
var borsh_1 = require("borsh");
var web3_js_1 = require("@solana/web3.js");
var spl_token_1 = require("@solana/spl-token");
var utils_1 = require("../../../common/utils");
function getTokenBurnInstructions(request) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var instructions, signers;
        return tslib_1.__generator(this, function (_c) {
            instructions = [];
            signers = [request.signer];
            instructions.push(createBurnTokenInstruction(request.mint, request.tokenAccount, (_a = request.owner) !== null && _a !== void 0 ? _a : request.signer.publicKey, (0, utils_1.alignBn)((0, utils_1.bigNumToBn)(request.amount), 8)));
            if (request.close) {
                instructions.push(spl_token_1.Token.createCloseAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, request.tokenAccount, request.signer.publicKey, (_b = request.owner) !== null && _b !== void 0 ? _b : request.signer.publicKey, []));
            }
            return [2 /*return*/, { instructions: instructions, signers: signers }];
        });
    });
}
exports.getTokenBurnInstructions = getTokenBurnInstructions;
function createBurnTokenInstruction(mint, tokenAccount, owner, amount) {
    var data = Buffer.from((0, borsh_1.serialize)(new Map([[
            Object,
            {
                kind: "struct",
                fields: [
                    ["instruction", "u8"],
                    ["amount", "u64"],
                ],
            },
        ]]), {
        instruction: 8,
        amount: (0, utils_1.bnToBuffer)(amount, "le", 8),
    }));
    var keys = [{
            pubkey: tokenAccount,
            isSigner: false,
            isWritable: true,
        }, {
            pubkey: mint,
            isSigner: false,
            isWritable: true,
        }];
    keys.push({
        pubkey: owner,
        isSigner: true,
        isWritable: false,
    });
    return new web3_js_1.TransactionInstruction({
        programId: spl_token_1.TOKEN_PROGRAM_ID,
        keys: keys,
        data: data,
    });
}
exports.createBurnTokenInstruction = createBurnTokenInstruction;
