"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTransferTokenInstruction = exports.getTokenTransferInstructions = void 0;
var tslib_1 = require("tslib");
var web3_js_1 = require("@solana/web3.js");
var borsh_1 = require("borsh");
var spl_token_1 = require("@solana/spl-token");
var mpl_core_1 = require("@metaplex-foundation/mpl-core");
var helpers_1 = require("../../../common/helpers");
var utils_1 = require("../../../common/utils");
function getTokenTransferInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var instructions, signers, destinationTokenAccount, _a;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    instructions = [];
                    signers = [request.signer];
                    return [4 /*yield*/, spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, request.mint, request.to)];
                case 1:
                    destinationTokenAccount = _b.sent();
                    _b.label = 2;
                case 2:
                    _b.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, mpl_core_1.Account.load(request.connection, destinationTokenAccount)];
                case 3:
                    _b.sent();
                    return [3 /*break*/, 5];
                case 4:
                    _a = _b.sent();
                    instructions.push((0, helpers_1.createAssociatedTokenAccountInstruction)(destinationTokenAccount, request.signer.publicKey, request.to, request.mint));
                    return [3 /*break*/, 5];
                case 5:
                    instructions.push(createTransferTokenInstruction(request.tokenAccount, destinationTokenAccount, request.signer.publicKey, (0, utils_1.alignBn)((0, utils_1.bigNumToBn)(request.amount), 8)));
                    return [2 /*return*/, { instructions: instructions, signers: signers }];
            }
        });
    });
}
exports.getTokenTransferInstructions = getTokenTransferInstructions;
function createTransferTokenInstruction(sourceTokenAccount, destinationTokenAccount, owner, amount) {
    var data = Buffer.from((0, borsh_1.serialize)(new Map([[
            Object,
            {
                kind: "struct",
                fields: [
                    ["instruction", "u8"],
                    ["amount", "u64"],
                ],
            },
        ]]), {
        instruction: 3,
        amount: (0, utils_1.bnToBuffer)(amount, "le", 8),
    }));
    var keys = [{
            pubkey: sourceTokenAccount,
            isSigner: false,
            isWritable: true,
        }, {
            pubkey: destinationTokenAccount,
            isSigner: false,
            isWritable: true,
        }];
    keys.push({
        pubkey: owner,
        isSigner: true,
        isWritable: false,
    });
    return new web3_js_1.TransactionInstruction({
        programId: spl_token_1.TOKEN_PROGRAM_ID,
        keys: keys,
        data: data,
    });
}
exports.createTransferTokenInstruction = createTransferTokenInstruction;
