"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaNftSdk = void 0;
var tslib_1 = require("tslib");
var prepared_transaction_1 = require("../prepared-transaction");
var mint_1 = require("./methods/mint");
var transfer_1 = require("./methods/transfer");
var burn_1 = require("./methods/burn");
var SolanaNftSdk = /** @class */ (function () {
    function SolanaNftSdk(connection, logger, accountSdk) {
        this.connection = connection;
        this.logger = logger;
        this.accountSdk = accountSdk;
    }
    SolanaNftSdk.prototype.mint = function (request) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var masterEditionSupply, amount, _a, mint, instructions;
            var _this = this;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if ("amount" in request) {
                            amount = request.amount;
                            masterEditionSupply = undefined;
                        }
                        else {
                            masterEditionSupply = request.masterEditionSupply;
                            amount = 1;
                        }
                        return [4 /*yield*/, (0, mint_1.getMintNftInstructions)(this.connection, request.signer, {
                                metadataLink: request.metadataUrl,
                                //mutableMetadata: true,
                                collection: request.collection,
                                masterEditionSupply: masterEditionSupply,
                                amount: amount,
                                verifyCreators: true,
                            })];
                    case 1:
                        _a = _b.sent(), mint = _a.mint, instructions = tslib_1.__rest(_a, ["mint"]);
                        return [2 /*return*/, {
                                tx: new prepared_transaction_1.PreparedTransaction(this.connection, instructions, request.signer, this.logger, function (tx) {
                                    _this.logger.log("NFT created ".concat(tx.txId));
                                    _this.logger.log("NFT: Mint Address is ".concat(mint.toString()));
                                }),
                                mint: mint,
                            }];
                }
            });
        });
    };
    SolanaNftSdk.prototype.transfer = function (request) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tokenAccount, _b, instructions;
            var _this = this;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (!((_a = request.tokenAccount) !== null && _a !== void 0)) return [3 /*break*/, 1];
                        _b = _a;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, this.accountSdk.getTokenAccountForMint({
                            owner: request.signer.publicKey,
                            mint: request.mint,
                        })];
                    case 2:
                        _b = _c.sent();
                        _c.label = 3;
                    case 3:
                        tokenAccount = _b;
                        if (!tokenAccount) {
                            throw new Error("Token account not specified");
                        }
                        return [4 /*yield*/, (0, transfer_1.getTokenTransferInstructions)({
                                connection: this.connection,
                                signer: request.signer,
                                tokenAccount: tokenAccount,
                                to: request.to,
                                mint: request.mint,
                                amount: request.amount,
                            })];
                    case 4:
                        instructions = _c.sent();
                        return [2 /*return*/, new prepared_transaction_1.PreparedTransaction(this.connection, instructions, request.signer, this.logger, function () {
                                _this.logger.log("".concat(request.amount.toString(), " token(s) ").concat(request.mint.toString(), " transferred to ").concat(request.to.toString()));
                            })];
                }
            });
        });
    };
    SolanaNftSdk.prototype.burn = function (request) {
        var _a, _b;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tokenAccount, _c, instructions;
            var _this = this;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        if (!((_a = request.tokenAccount) !== null && _a !== void 0)) return [3 /*break*/, 1];
                        _c = _a;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, this.accountSdk.getTokenAccountForMint({
                            owner: (_b = request.owner) !== null && _b !== void 0 ? _b : request.signer.publicKey,
                            mint: request.mint,
                        })];
                    case 2:
                        _c = _d.sent();
                        _d.label = 3;
                    case 3:
                        tokenAccount = _c;
                        if (!tokenAccount) {
                            throw new Error("Token account not specified");
                        }
                        return [4 /*yield*/, (0, burn_1.getTokenBurnInstructions)({
                                connection: this.connection,
                                signer: request.signer,
                                tokenAccount: tokenAccount,
                                mint: request.mint,
                                amount: request.amount,
                                owner: request.owner,
                                close: request.closeAssociatedAccount,
                            })];
                    case 4:
                        instructions = _d.sent();
                        return [2 /*return*/, new prepared_transaction_1.PreparedTransaction(this.connection, instructions, request.signer, this.logger, function () {
                                _this.logger.log("".concat(request.amount.toString(), " token(s) ").concat(request.mint.toString(), " burned"));
                            })];
                }
            });
        });
    };
    return SolanaNftSdk;
}());
exports.SolanaNftSdk = SolanaNftSdk;
