"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getActionHouseBuyInstructions = void 0;
var tslib_1 = require("tslib");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var solana_wallet_1 = require("@rarible/solana-wallet");
var spl_token_1 = require("@solana/spl-token");
var mpl_auction_house_1 = require("@metaplex-foundation/mpl-auction-house");
var contracts_1 = require("../../../common/contracts");
var helpers_1 = require("../../../common/helpers");
var auction_house_helpers_1 = require("../../../common/auction-house-helpers");
var utils_1 = require("../../../common/utils");
function getActionHouseBuyInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var walletKeyPair, anchorProgram, auctionHouseObj, buyPriceAdjusted, tokenSizeAdjusted, _a, escrowPaymentAccount, escrowBump, tokenAccountKey, tla, _b, tradeState, tradeBump, isNative, ata, transferAuthority, signers, instruction, _c, _d, instructions;
        var _e;
        return tslib_1.__generator(this, function (_f) {
            switch (_f.label) {
                case 0:
                    walletKeyPair = request.signer;
                    return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(request.connection, request.signer)];
                case 1:
                    anchorProgram = _f.sent();
                    return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                case 2:
                    auctionHouseObj = _f.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.price), auctionHouseObj.treasuryMint, walletKeyPair)];
                case 3:
                    buyPriceAdjusted = _f.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.tokensAmount), request.mint, walletKeyPair)];
                case 4:
                    tokenSizeAdjusted = _f.sent();
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseBuyerEscrow)(request.auctionHouse, walletKeyPair.publicKey)];
                case 5:
                    _a = tslib_1.__read.apply(void 0, [_f.sent(), 2]), escrowPaymentAccount = _a[0], escrowBump = _a[1];
                    if (!request.tokenAccount) return [3 /*break*/, 6];
                    tokenAccountKey = request.tokenAccount;
                    return [3 /*break*/, 8];
                case 6: return [4 /*yield*/, anchorProgram.provider.connection.getTokenLargestAccounts(request.mint)];
                case 7:
                    tla = _f.sent();
                    tokenAccountKey = tla.value[0].address;
                    _f.label = 8;
                case 8: return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseTradeState)(request.auctionHouse, walletKeyPair.publicKey, tokenAccountKey, auctionHouseObj.treasuryMint, request.mint, tokenSizeAdjusted, buyPriceAdjusted)];
                case 9:
                    _b = tslib_1.__read.apply(void 0, [_f.sent(), 2]), tradeState = _b[0], tradeBump = _b[1];
                    isNative = auctionHouseObj.treasuryMint.equals(contracts_1.WRAPPED_SOL_MINT);
                    return [4 /*yield*/, (0, helpers_1.getAssociatedTokenAccountForMint)(auctionHouseObj.treasuryMint, walletKeyPair.publicKey)];
                case 10:
                    ata = (_f.sent())[0];
                    transferAuthority = solana_wallet_1.SolanaKeypairWallet.generate();
                    signers = isNative ? [] : [transferAuthority];
                    _d = (_c = mpl_auction_house_1.AuctionHouseProgram.instructions).createBuyInstruction;
                    _e = {
                        wallet: request.signer.publicKey,
                        paymentAccount: isNative ? walletKeyPair.publicKey : ata,
                        transferAuthority: isNative ? walletKeyPair.publicKey : transferAuthority.publicKey,
                        treasuryMint: auctionHouseObj.treasuryMint,
                        tokenAccount: tokenAccountKey
                    };
                    return [4 /*yield*/, (0, helpers_1.getMetadata)(request.mint)];
                case 11:
                    instruction = _d.apply(_c, [(_e.metadata = _f.sent(),
                            _e.escrowPaymentAccount = escrowPaymentAccount,
                            _e.authority = auctionHouseObj.authority,
                            _e.auctionHouse = request.auctionHouse,
                            _e.auctionHouseFeeAccount = auctionHouseObj.auctionHouseFeeAccount,
                            _e.buyerTradeState = tradeState,
                            _e), {
                            tradeStateBump: tradeBump,
                            escrowPaymentBump: escrowBump,
                            buyerPrice: (0, utils_1.bigNumToBn)(buyPriceAdjusted),
                            tokenSize: (0, utils_1.bigNumToBn)(tokenSizeAdjusted),
                        }]);
                    if (!isNative) {
                        instruction.keys
                            .filter(function (k) { return k.pubkey.equals(transferAuthority.publicKey); })
                            .map(function (k) { return (k.isSigner = true); });
                    }
                    instructions = tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read((isNative
                        ? []
                        : [
                            spl_token_1.Token.createApproveInstruction(spl_token_1.TOKEN_PROGRAM_ID, ata, transferAuthority.publicKey, walletKeyPair.publicKey, [], buyPriceAdjusted.toNumber()),
                        ])), false), [
                        instruction
                    ], false), tslib_1.__read((isNative
                        ? []
                        : [
                            spl_token_1.Token.createRevokeInstruction(spl_token_1.TOKEN_PROGRAM_ID, ata, walletKeyPair.publicKey, []),
                        ])), false);
                    return [2 /*return*/, { instructions: instructions, signers: signers }];
            }
        });
    });
}
exports.getActionHouseBuyInstructions = getActionHouseBuyInstructions;
