"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAuctionHouseCancelInstructions = void 0;
var tslib_1 = require("tslib");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var mpl_auction_house_1 = require("@metaplex-foundation/mpl-auction-house");
var auction_house_helpers_1 = require("../../../common/auction-house-helpers");
var helpers_1 = require("../../../common/helpers");
var utils_1 = require("../../../common/utils");
function getAuctionHouseCancelInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var anchorProgram, auctionHouseObj, buyPriceAdjusted, tokenSizeAdjusted, tla, tokenAccountKey, _a, tradeState, instruction;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(request.connection, request.signer)];
                case 1:
                    anchorProgram = _b.sent();
                    return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                case 2:
                    auctionHouseObj = _b.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.price), auctionHouseObj.treasuryMint, request.signer)];
                case 3:
                    buyPriceAdjusted = _b.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.tokensAmount), request.mint, request.signer)];
                case 4:
                    tokenSizeAdjusted = _b.sent();
                    return [4 /*yield*/, anchorProgram.provider.connection.getTokenLargestAccounts(request.mint)];
                case 5:
                    tla = _b.sent();
                    if (tla.value.length === 0) {
                        throw new Error("The Mint(NFT, Tokens) largest token account can't be found, this could be " +
                            "network instability or you have the wrong mint address.");
                    }
                    tokenAccountKey = tla.value[0].address;
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseTradeState)(request.auctionHouse, request.signer.publicKey, tokenAccountKey, auctionHouseObj.treasuryMint, request.mint, tokenSizeAdjusted, buyPriceAdjusted)];
                case 6:
                    _a = tslib_1.__read.apply(void 0, [_b.sent(), 1]), tradeState = _a[0];
                    instruction = mpl_auction_house_1.AuctionHouseProgram.instructions.createCancelInstruction({
                        wallet: request.signer.publicKey,
                        tokenAccount: tokenAccountKey,
                        tokenMint: request.mint,
                        authority: auctionHouseObj.authority,
                        auctionHouse: request.auctionHouse,
                        auctionHouseFeeAccount: auctionHouseObj.auctionHouseFeeAccount,
                        tradeState: tradeState,
                    }, {
                        buyerPrice: (0, utils_1.bigNumToBn)(buyPriceAdjusted),
                        tokenSize: (0, utils_1.bigNumToBn)(tokenSizeAdjusted),
                    });
                    return [2 /*return*/, { instructions: [instruction], signers: [] }];
            }
        });
    });
}
exports.getAuctionHouseCancelInstructions = getAuctionHouseCancelInstructions;
