"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAuctionHouseSellInstructions = void 0;
var tslib_1 = require("tslib");
var bignumber_js_1 = tslib_1.__importDefault(require("bignumber.js"));
var mpl_auction_house_1 = require("@metaplex-foundation/mpl-auction-house");
var helpers_1 = require("../../../common/helpers");
var auction_house_helpers_1 = require("../../../common/auction-house-helpers");
var utils_1 = require("../../../common/utils");
function getAuctionHouseSellInstructions(request) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var anchorProgram, auctionHouseObj, buyPriceAdjusted, tokenSizeAdjusted, tokenAccountKey, _a, programAsSigner, programAsSignerBump, _b, tradeState, tradeBump, _c, freeTradeState, freeTradeBump, signers, instruction, _d, _e;
        var _f;
        return tslib_1.__generator(this, function (_g) {
            switch (_g.label) {
                case 0: return [4 /*yield*/, (0, auction_house_helpers_1.loadAuctionHouseProgram)(request.connection, request.signer)];
                case 1:
                    anchorProgram = _g.sent();
                    return [4 /*yield*/, anchorProgram.account.auctionHouse.fetch(request.auctionHouse)];
                case 2:
                    auctionHouseObj = _g.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.price), auctionHouseObj.treasuryMint, request.signer)];
                case 3:
                    buyPriceAdjusted = _g.sent();
                    return [4 /*yield*/, (0, helpers_1.getPriceWithMantissa)(request.connection, new bignumber_js_1.default(request.tokensAmount), request.mint, request.signer)];
                case 4:
                    tokenSizeAdjusted = _g.sent();
                    return [4 /*yield*/, (0, helpers_1.getAssociatedTokenAccountForMint)(request.mint, request.signer.publicKey)];
                case 5:
                    tokenAccountKey = (_g.sent())[0];
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseProgramAsSigner)()];
                case 6:
                    _a = tslib_1.__read.apply(void 0, [_g.sent(), 2]), programAsSigner = _a[0], programAsSignerBump = _a[1];
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseTradeState)(request.auctionHouse, request.signer.publicKey, tokenAccountKey, auctionHouseObj.treasuryMint, request.mint, tokenSizeAdjusted, buyPriceAdjusted)];
                case 7:
                    _b = tslib_1.__read.apply(void 0, [_g.sent(), 2]), tradeState = _b[0], tradeBump = _b[1];
                    return [4 /*yield*/, (0, auction_house_helpers_1.getAuctionHouseTradeState)(request.auctionHouse, request.signer.publicKey, tokenAccountKey, auctionHouseObj.treasuryMint, request.mint, tokenSizeAdjusted, new bignumber_js_1.default(0))];
                case 8:
                    _c = tslib_1.__read.apply(void 0, [_g.sent(), 2]), freeTradeState = _c[0], freeTradeBump = _c[1];
                    signers = [];
                    _e = (_d = mpl_auction_house_1.AuctionHouseProgram.instructions).createSellInstruction;
                    _f = {
                        wallet: request.signer.publicKey,
                        tokenAccount: tokenAccountKey
                    };
                    return [4 /*yield*/, (0, helpers_1.getMetadata)(request.mint)];
                case 9:
                    instruction = _e.apply(_d, [(_f.metadata = _g.sent(),
                            _f.authority = auctionHouseObj.authority,
                            _f.auctionHouse = request.auctionHouse,
                            _f.auctionHouseFeeAccount = auctionHouseObj.auctionHouseFeeAccount,
                            _f.sellerTradeState = tradeState,
                            _f.freeSellerTradeState = freeTradeState,
                            _f.programAsSigner = programAsSigner,
                            _f), {
                            tradeStateBump: tradeBump,
                            freeTradeStateBump: freeTradeBump,
                            programAsSignerBump: programAsSignerBump,
                            buyerPrice: (0, utils_1.bigNumToBn)(buyPriceAdjusted),
                            tokenSize: (0, utils_1.bigNumToBn)(tokenSizeAdjusted),
                        }]);
                    instruction.keys
                        .filter(function (k) { return k.pubkey.equals(request.signer.publicKey); })
                        .map(function (k) { return (k.isSigner = true); });
                    return [2 /*return*/, { instructions: [instruction], signers: signers }];
            }
        });
    });
}
exports.getAuctionHouseSellInstructions = getAuctionHouseSellInstructions;
