"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.postFillAction = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const web3_ethereum_1 = require("@rarible/web3-ethereum");
const protocol_ethereum_sdk_1 = require("@rarible/protocol-ethereum-sdk");
const address_1 = require("@rarible/types/build/address");
const utils_1 = require("../utils");
function postFillAction(req, res) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!req.body.request || !req.body.from) {
                return res.status(400)
                    .json({ message: "body.request or body.from has been missed" });
            }
            const { blockchain, address: from } = (0, utils_1.getAddressParts)(req.body.from);
            if (from === undefined) {
                return res.status(400)
                    .json({ message: "body.from should be in union format" });
            }
            switch (blockchain) {
                case "ETHEREUM":
                    return ethereumPostFillAction(from, req.body.request, res);
                default:
                    return res.status(400)
                        .json({ message: "Unsupported blockchain " + blockchain });
            }
        }
        catch (e) {
            console.error(e);
            return res.status(500)
                .json({ message: "Internal error" });
        }
    });
}
exports.postFillAction = postFillAction;
function ethereumPostFillAction(from, request, res) {
    return __awaiter(this, void 0, void 0, function* () {
        const web3Ethereum = new web3_ethereum_1.Web3Ethereum({ web3: utils_1.web3Provider, from });
        const sdk = (0, protocol_ethereum_sdk_1.createRaribleSdk)(web3Ethereum, process.env.SDK_ENV, {
            apiClientParams: {
                fetchApi: node_fetch_1.default,
            },
        });
        if (request.orderId) {
            const { address: orderId } = (0, utils_1.getAddressParts)(request.orderId);
            if (!orderId) {
                return res.status(400)
                    .json({ message: "body.request.orderId should be in union format" });
            }
            request.order = yield sdk.apis.order.getOrderByHash({
                hash: orderId,
            });
        }
        const txData = yield sdk.order.getBuyTxData({ from: (0, address_1.toAddress)(from), request });
        return res.json(txData);
    });
}
//# sourceMappingURL=fill.js.map