"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_1 = __importDefault(require("web3"));
const ethereum_sdk_test_common_1 = require("@rarible/ethereum-sdk-test-common");
const web3_ethereum_1 = require("@rarible/web3-ethereum");
const protocol_ethereum_sdk_1 = require("@rarible/protocol-ethereum-sdk");
const mint_1 = require("@rarible/protocol-ethereum-sdk/build/common/mint");
const mint_2 = require("@rarible/protocol-ethereum-sdk/build/nft/mint");
const retry_1 = require("@rarible/protocol-ethereum-sdk/build/common/retry");
// @ts-ignore
const supertest_1 = __importDefault(require("supertest"));
const app_1 = require("../app");
describe.skip("get buy transaction", () => {
    const { provider } = (0, ethereum_sdk_test_common_1.createE2eProvider)("ded057615d97f0f1c751ea2795bc4b03bbf44844c13ab4f5e6fd976506c276b9");
    const web3Seller = new web3_1.default(provider);
    const ethereum1 = new web3_ethereum_1.Web3Ethereum({ web3: web3Seller, gas: 1000000 });
    const sdkItemOwner = (0, protocol_ethereum_sdk_1.createRaribleSdk)(ethereum1, process.env.SDK_ENV);
    const erc721Address = "0x797cb2325e762e6178452b1D7478Aa616aA1656E";
    const buyerPk = "26250bb39160076f030517503da31e11aca80060d14f84ebdaced666efb89e21";
    const { provider: providerBuyer } = (0, ethereum_sdk_test_common_1.createE2eProvider)(buyerPk);
    const web3Buyer = new web3_1.default(providerBuyer);
    it("get buy transaction and send from buyer", () => __awaiter(void 0, void 0, void 0, function* () {
        const itemOwnerAddress = yield ethereum1.getFrom();
        const [buyerAddress] = yield web3Buyer.eth.getAccounts();
        const mintResult = yield sdkItemOwner.nft.mint({
            uri: "ipfs://ipfs/hash",
            creators: [{
                    account: itemOwnerAddress,
                    value: 10000,
                }],
            lazy: false,
            collection: (0, mint_1.createErc721V3Collection)(erc721Address),
        });
        if (mintResult.type === mint_2.MintResponseTypeEnum.ON_CHAIN) {
            yield mintResult.transaction.wait();
        }
        yield (0, retry_1.retry)(10, 2000, () => {
            return sdkItemOwner.apis.nftItem.getNftItemById({ itemId: mintResult.itemId });
        });
        const sellOrder = yield sdkItemOwner.order.sell({
            type: "DATA_V2",
            makeAssetType: {
                assetClass: "ERC721",
                contract: erc721Address,
                tokenId: mintResult.tokenId,
            },
            amount: 1,
            takeAssetType: {
                assetClass: "ETH",
            },
            priceDecimal: "0.000000000000000002",
            payouts: [],
            originFees: [],
        });
        const response = yield (0, supertest_1.default)(app_1.app)
            .post("/v0.1/orders/fill-tx")
            .send({
            from: "ETHEREUM:" + buyerAddress,
            request: {
                order: sellOrder,
                amount: 1,
                originFees: [],
                payouts: [],
            },
        })
            .expect(200);
        const buyerNonce = yield web3Buyer.eth.getTransactionCount(buyerAddress, "latest"); // nonce starts counting from 0
        const buyerTxData = Object.assign(Object.assign({}, response.body), { nonce: buyerNonce, gasLimit: 200000 });
        const signedBuyerTx = yield web3Buyer.eth.accounts.signTransaction(buyerTxData, buyerPk);
        if (!signedBuyerTx.rawTransaction) {
            throw new Error("Raw transaction is empty");
        }
        yield web3Buyer.eth.sendSignedTransaction(signedBuyerTx.rawTransaction);
        yield (0, retry_1.retry)(5, 2000, () => __awaiter(void 0, void 0, void 0, function* () {
            yield sdkItemOwner.apis.nftOwnership.getNftOwnershipById({
                ownershipId: `${mintResult.itemId}:${buyerAddress}`,
            });
        }));
    }));
    it("get buy transaction with orderId and send from buyer", () => __awaiter(void 0, void 0, void 0, function* () {
        const itemOwnerAddress = yield ethereum1.getFrom();
        const [buyerAddress] = yield web3Buyer.eth.getAccounts();
        const mintResult = yield sdkItemOwner.nft.mint({
            uri: "ipfs://ipfs/hash",
            creators: [{
                    account: itemOwnerAddress,
                    value: 10000,
                }],
            lazy: false,
            collection: (0, mint_1.createErc721V3Collection)(erc721Address),
        });
        if (mintResult.type === mint_2.MintResponseTypeEnum.ON_CHAIN) {
            yield mintResult.transaction.wait();
        }
        yield (0, retry_1.retry)(10, 2000, () => {
            return sdkItemOwner.apis.nftItem.getNftItemById({ itemId: mintResult.itemId });
        });
        const sellOrder = yield sdkItemOwner.order.sell({
            type: "DATA_V2",
            makeAssetType: {
                assetClass: "ERC721",
                contract: erc721Address,
                tokenId: mintResult.tokenId,
            },
            amount: 1,
            takeAssetType: {
                assetClass: "ETH",
            },
            priceDecimal: "0.000000000000000002",
            payouts: [],
            originFees: [],
        });
        const response = yield (0, supertest_1.default)(app_1.app)
            .post("/v0.1/orders/fill-tx")
            .send({
            from: "ETHEREUM:" + buyerAddress,
            request: {
                orderId: "ETHEREUM:" + sellOrder.hash,
                amount: 1,
                originFees: [],
                payouts: [],
            },
        })
            .expect(200);
        const buyerNonce = yield web3Buyer.eth.getTransactionCount(buyerAddress, "latest"); // nonce starts counting from 0
        const buyerTxData = Object.assign(Object.assign({}, response.body), { nonce: buyerNonce, gasLimit: 200000 });
        const signedBuyerTx = yield web3Buyer.eth.accounts.signTransaction(buyerTxData, buyerPk);
        if (!signedBuyerTx.rawTransaction) {
            throw new Error("Raw transaction is empty");
        }
        yield web3Buyer.eth.sendSignedTransaction(signedBuyerTx.rawTransaction);
        yield (0, retry_1.retry)(40, 2000, () => __awaiter(void 0, void 0, void 0, function* () {
            yield sdkItemOwner.apis.nftOwnership.getNftOwnershipById({
                ownershipId: `${mintResult.itemId}:${buyerAddress}`,
            });
        }));
    }));
});
//# sourceMappingURL=fill.test.js.map