"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainEthereumTransaction = void 0;
var tslib_1 = require("tslib");
var api_client_1 = require("@rarible/api-client");
var BlockchainEthereumTransaction = /** @class */ (function () {
    function BlockchainEthereumTransaction(transaction, network, resultExtractor) {
        this.transaction = transaction;
        this.network = network;
        this.resultExtractor = resultExtractor;
        this.blockchain = this.getBlockchain(network);
    }
    BlockchainEthereumTransaction.prototype.getBlockchain = function (network) {
        switch (network) {
            case "mumbai":
            case "polygon":
                return api_client_1.Blockchain.POLYGON;
            default:
                return api_client_1.Blockchain.ETHEREUM;
        }
    };
    BlockchainEthereumTransaction.prototype.hash = function () {
        return this.transaction.hash;
    };
    BlockchainEthereumTransaction.prototype.wait = function () {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var receipt;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.transaction.wait()];
                    case 1:
                        receipt = _b.sent();
                        return [2 /*return*/, {
                                blockchain: this.blockchain,
                                hash: this.transaction.hash,
                                result: (_a = this.resultExtractor) === null || _a === void 0 ? void 0 : _a.call(this, receipt),
                            }];
                }
            });
        });
    };
    BlockchainEthereumTransaction.prototype.getTxLink = function () {
        switch (this.network) {
            case "mainnet":
                return "https://etherscan.io/tx/".concat(this.hash());
            case "mumbai":
                return "https://mumbai.polygonscan.com/tx/".concat(this.hash());
            case "polygon":
                return "https://polygonscan.com/tx/".concat(this.hash());
            case "testnet":
                return "https://rinkeby.etherscan.io/tx/".concat(this.hash());
            default:
                throw new Error("Unsupported transaction network");
        }
    };
    Object.defineProperty(BlockchainEthereumTransaction.prototype, "isEmpty", {
        get: function () {
            return false;
        },
        enumerable: false,
        configurable: true
    });
    return BlockchainEthereumTransaction;
}());
exports.BlockchainEthereumTransaction = BlockchainEthereumTransaction;
